import config from "config";
import client from "lib/client";
import { getRandomInt } from "lib/utils";
import {
	customContent,
	greetContent,
	sleepContent,
	createEmbed,
} from "./greeting.components.ts";
import {
	type ChatInputCommandInteraction,
	Client,
	EmbedBuilder,
	type CacheType,
	type GuildMember,
	type Interaction,
} from "discord.js";
import { DmService } from "actions/dm/dm.service.ts";

export class GreetingService {
	dmService: DmService;

	constructor() {
		this.dmService = new DmService();
	}

	async handleInteraction(
		interaction: Interaction<CacheType>
	) {
		console.log("accept");

		if (interaction.isChatInputCommand()) {
			await this.acceptUser(interaction);
		}
	}

	async custom() {
		console.log("custom message");

		client.users.send(config.discord.myId, "hat funktioniert :)");
	}

	async welcome(member: GuildMember) {
		console.log("welcome msg");

		const welcomeContents = [
			`willkommen auf dem server, ${member}! 💕`,
			`hey ${member}! schoen, dass du hier bist! 😊`,
			`hi ${member}, willkommen! viel spass hier! 💖`,
			`willkommen, ${member}! schoen, dass du da bist! 🥳`,
			`moin ${member}! viel spass im server! c:`,
			`hey ${member}, herzlich willkommen! fuehl dich wie zu hause! <3`,
			`hi ${member}! cool, dass du da bist! 👏`,
			`willkommen, ${member}! wir freuen uns, dass du hier bist! 💕`,
			`hey ${member}! schoen, dass du bei uns bist! :3`,
			`willkommen auf dem server, ${member}! viel spass hier! ✨`,
		];
		const welcomeContent =
			welcomeContents[Math.floor(Math.random() * welcomeContents.length)];

		try {
			const channels = client.channels;
			const channel = channels.cache.get(config.discord.channelIdWelcome);

			if (channel?.isTextBased() && channel?.isSendable()) {
				await channel.send(welcomeContent);
			}
		} catch (error) {
			console.error("error while sending a welcome msg:", error);
		}
	}

	async acceptUser(
		interaction: ChatInputCommandInteraction<CacheType>
	) {
		console.log("accept user");

		// get the string option
		const input = interaction.options.getString("input") || "";
		// return the value
		//console.log(input);

		// permission check
		const userIdCommand = interaction.user.id;
		if (userIdCommand !== config.discord.myId) {
			await interaction.reply({
				content: "you have no permission for that command",
				ephemeral: true,
			});
			return;
		}

		try {
			// get user id from mentioning
			const userId = input.replace(/[<@!>]/g, "");
			console.log(userId.toString());
			const guild = interaction.guild;
			if (!guild) {
				await interaction.reply({
					content: "command can only be used on a server",
					ephemeral: true,
				});
				return;
			}

			const username = (await guild.members.fetch(userId)).user.username;
			console.log(username);
			// get member from id
			const member = await guild.members.fetch(userId);

			const role = config.discord.roleStudy;

			await member.roles.add(role);

			await interaction.reply({
				content: `die rolle *lernende:r* wurde erfolgreich an ${member.user.username} vergeben`,
				ephemeral: true,
			});

			this.dmService.acceptDm(member);
		} catch (error) {
			console.error("Fehler beim Hinzufügen der Rolle:", error);
			await interaction.reply({
				content:
					"Es gab einen Fehler beim Hinzufügen der Rolle. Stelle sicher, dass du einen gültigen User erwähnt hast.",
				ephemeral: true,
			});
		}
	}

	async greet() {
		client.user?.setActivity("guten morgen! :3", { type: 4 });
		console.log("set activity: awake");
		client.user?.setPresence({
			status: "online",
		});

		const channels = client.channels;

		const channel = channels.cache.get(config.discord.testChannel);

		if (channel?.isTextBased && channel?.isSendable()) {
			await channel.send({ content: this.getContent(false) });
		}
	}

	async sleep() {
		client.user?.setActivity("zzzzZZ..", { type: 4 });
		console.log("set activity: asleep");
		client.user?.setPresence({
			status: "dnd",
		});

		const channels = client.channels;

		const channel = channels.cache.get(config.discord.testChannel);

		if (channel?.isTextBased && channel?.isSendable()) {
			await channel.send({ content: this.getContent(true) });
		}
	}

	async newYear() {
		client.user?.setActivity("frohes neues! :)", { type: 4 });
		console.log("set activity: happy new Year");
		client.user?.setPresence({
			status: "online",
		});

		/*const channels = client.channels;

		const channel = channels.cache.get(config.discord.channelId);

		if (channel?.isTextBased && channel?.isSendable()) {
			await channel.send({ content: "frohes neues! @everyone" });
		}*/
	}

	getContent(asleep: boolean) {
		if (asleep) {
			return sleepContent[getRandomInt(0, sleepContent.length - 1)];
		}
		return greetContent[getRandomInt(0, greetContent.length - 1)];
	}
}
